/**
 * @file err_lookup.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#include <string>
#include <vector>


#ifndef ASSERT
#ifdef _DEBUG
#include <assert.h>
#define ASSERT(a) assert(a)
#define VERIFY(a) assert(a)
#else //_DEBUG
#define ASSERT(a)	((void)0)
#define VERIFY(a)	((void)(a))
#endif //_DEBUG
#endif //ASSERT



#define ERRTABLE_ADDITEM(name, fmt)\
ifdef fmt\
	gErrorTable.Add(IDS_TXPLN_##name##, IDS_TXFMT_##fmt##}\
endif

struct ErrorMessage
{
	ErrorMessage(DWORD ErrorCode, const std::wstring &BasicText, const std::wstring &FormattedText) : Code(ErrorCode), BasicText(BasicText), FormattedText(FormattedText) {};
	DWORD Code;
	std::wstring BasicText;
	std::wstring FormattedText;
};

class ErrorLookup
{
	HMODULE ResHandle;
	std::vector<ErrorMessage> ErrorTable;

	void LoadResStr(UINT ID, std::wstring &Str);

public:

	ErrorLookup();

	void Init(HMODULE ModuleHandle);

	void Add(DWORD ErrorCode, UINT IdsBasic, UINT IdsFormatted);

	ErrorMessage *FindErrorCode(DWORD ErrorCode);

#if _DEBUG
	void DumpToFile(LPCSTR FileName);
#endif
};


extern ErrorLookup gErrorTable;


std::wstring GetErrorMessage(DWORD ErrorCode);

std::wstring GetErrorMessage(DWORD ErrorCode, LPCWSTR Param1, LPCWSTR Param2 = NULL, LPCWSTR Param3 = NULL, LPCWSTR Param4 = NULL, LPCWSTR Param5 = NULL);

std::wstring MakeErrorMessage(std::wstring Message, LPCWSTR Param1 = NULL, LPCWSTR Param2 = NULL, LPCWSTR Param3 = NULL, LPCWSTR Param4 = NULL, LPCWSTR Param5 = NULL);
