/**
 * @file EracmdExceptions.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "EracmdExceptions.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/era_api/err_lookup.h>
#include "commands/Command_RunScript.h"
#include "util/stringUtils.h"
#include "ScriptInterpreter.h"

extern ConsoleColors_Win gConsoleColors;

EraCMDException::EraCMDException()
:
	errorCode(0)
{
}

EraCMDException::EraCMDException(DWORD ErrorCode, LPCWSTR Param1, LPCWSTR Param2, LPCWSTR Param3, LPCWSTR Param4, LPCWSTR Param5)
:
	errorCode(ErrorCode),
	message(::GetErrorMessage(ErrorCode, Param1, Param2, Param3, Param4, Param5)),
	fileName(Command_RunScript::GetCurrentFileName()),
	command(ScriptInterpreter::lastCommand)
{
}

EraCMDException_Pause::EraCMDException_Pause()
:
	EraCMDException()
{
}

EraCMDException_Pause::EraCMDException_Pause(DWORD ErrorCode, LPCWSTR Param1, LPCWSTR Param2, LPCWSTR Param3, LPCWSTR Param4, LPCWSTR Param5)
:
	EraCMDException(ErrorCode, Param1, Param2, Param3, Param4, Param5)
{

}

void EraCMDException::Print(std::wostream &os) const
{
	os << gConsoleColors.LIGHT_RED << LoadStringFromResource(IDS_ERROR_UPPERCASE) << " " << errorCode << ": " << gConsoleColors.DEFAULT << message << std::endl;

	if (!command.empty())
		os << gConsoleColors.LIGHT_YELLOW << LoadStringFromResource(IDS_COMMAND) << ": " << gConsoleColors.DEFAULT << command << std::endl;
	if (!fileName.empty())
		os << gConsoleColors.LIGHT_YELLOW << LoadStringFromResource(IDS_FILE) << ": " << gConsoleColors.DEFAULT << fileName << std::endl;
}

EraCMDException_FromApi::EraCMDException_FromApi(const uint32_t resultFromApi)
:	EraCMDException(resultFromApi)
{
	const wchar_t *lastErrorMessageFromApi;
	era_GetCurrentThreadLastErrorMessage(&lastErrorMessageFromApi);
	message = lastErrorMessageFromApi;
}

EraCMDException_FromApi_Pause::EraCMDException_FromApi_Pause(DWORD resultFromApi)
: EraCMDException_FromApi(resultFromApi)
{
}

