/**
 * @file FileLoader.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "FileLoader.h"

#include <Common/era_api/err_codes.h>
#include "EracmdExceptions.h"

FileLoader::FileLoader(const std::wstring &path)
:
	path(path)
{
	bomBytes_utf16.push_back((char)0xFF);
	bomBytes_utf16.push_back((char)0xFE);

	bomBytes_utf8.push_back((char)0xEF);
	bomBytes_utf8.push_back((char)0xBB);
	bomBytes_utf8.push_back((char)0xBF);
}

std::wstring FileLoader::Load() const
{
	std::ifstream is(path.c_str(), std::ios::binary);
	std::vector<char> bytes((std::istreambuf_iterator<char>(is)), std::istreambuf_iterator<char>());
	if (is.fail())
	{
		throw EraCMDException(CMDERROR_UNABLE_TO_READ_FILE, path.c_str());
	}

	if (HasBom(bomBytes_utf16, bytes))
	{
		std::vector<char>::size_type length = bytes.size() - bomBytes_utf16.size();
		if (length == 0)
		{
			return std::wstring();
		}

		const char *begin = &bytes[bomBytes_utf16.size()];

		return std::wstring((wchar_t *)begin, length / 2);
	}
	else if (HasBom(bomBytes_utf8, bytes))
	{
		std::vector<char>::size_type length = bytes.size() - bomBytes_utf8.size();
		if (length == 0)
		{
			return std::wstring();
		}

		const char *begin = &bytes[bomBytes_utf8.size()];

		return LoadFromMultiByte(CP_UTF8, begin, length);
	}
	else
	{
		std::vector<char>::size_type length = bytes.size();
		if (length == 0)
		{
			return std::wstring();
		}

		const char *begin = &bytes[0];

		return LoadFromMultiByte(CP_ACP, begin, length);
	}
}

std::wstring FileLoader::LoadFromMultiByte(const unsigned int codePage, const char *str, size_t length) const
{
	if (length == 0)
	{
		return std::wstring();
	}
	else
	{
		const int resultLength = ::MultiByteToWideChar(codePage, 0, str, (int) length, NULL, 0);
		if (resultLength == 0)
		{
			throw EraCMDException(CMDERROR_ENCODING_CONVERSION_FAILED, path.c_str());
		}

		std::wstring result;
		result.resize(resultLength);

		if (!::MultiByteToWideChar(codePage, 0, str, (int) length, &result[0], resultLength))
		{
			throw EraCMDException(CMDERROR_ENCODING_CONVERSION_FAILED, path.c_str());
		}

		return result;
	}
}

bool FileLoader::HasBom(const std::vector<char> &bomBytes, const std::vector<char> &fileBytes) const
{
	if (fileBytes.size() < bomBytes.size())
	{
		return false;
	}

	for (size_t i = 0; i < bomBytes.size(); ++i)
	{
		if (fileBytes[i] != bomBytes[i])
		{
			return false;
		}
	}

	return true;
}

