/**
 * @file PolicyTree.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#include <Common/era_api/PolicyBaseInfo.h>

struct PolicyTreeInfo : public PolicyBaseInfo
{
	PolicyTreeInfo();
	PolicyTreeInfo(int nId, const std::wstring &policyId, const std::wstring &policyName, const std::wstring &policyPrimaryServer, DWORD dwFlags, int nPrimaryId, int nParentPrimaryId, const std::wstring &parentPrimaryServer, const std::wstring &repDirectParent);
	PolicyTreeInfo(int nId, const wchar_t *policyId, const wchar_t *policyName, const wchar_t *policyPrimaryServer, DWORD dwFlags, int nPrimaryId, int nParentPrimaryId, const wchar_t *parentPrimaryServer, const wchar_t *repDirectParent);

	int nParentIndex;
};

template<typename T>
class TreeItem
{
public:
	TreeItem(const T &data)
		: data(data)
	{}
	void AddChild(std::shared_ptr<TreeItem<T> > &child)
	{
		children.push_back(child);
	}

	const T &data;
	std::vector<std::shared_ptr<TreeItem<T> > > children;
};

typedef TreeItem<PolicyTreeInfo> PolicyTree;
typedef std::shared_ptr<PolicyTree> PolicyTreePtr;

class PolicyTreeUtils
{
public:
	static void PrintPolicyTree(PolicyTreePtr parent, std::wstring namePrefix, std::vector<StringVector> &rows, bool printFirstRoot);
	static void BuildPolicyTree(std::vector<PolicyTreeInfo> &policies, PolicyTreePtr &rootItem, const wchar_t *onlyPolicyOfServer, const wchar_t *primaryServer);
private:
	static PolicyTreePtr AttachToTree(std::vector<PolicyTreeInfo> &policies, PolicyTreePtr parent, size_t nArrayChildIndex, const wchar_t *onlyPolicyOfServer, const wchar_t *primaryServer);
};
