/**
 * @file ScriptInterpreter.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#include "ConnectionParams.h"

class ParamValidator;
class AutocompleteCollector;
class Command;
class CommandFlagsStack;

class CommandList
{
public:
	void Add(std::wstring commandString, std::shared_ptr<Command> command);
	std::shared_ptr<Command> GetCommand(elementArray &elements /* could be more words, returns first matching command, removes relevant elements */, std::wstring &identifiedString);
	std::shared_ptr<Command> GetCommand(const std::wstring &commandString);
	void GetCommandStrings(std::vector<std::wstring> &outCommandList) const;
private:
	std::vector< std::pair<std::wstring, std::shared_ptr<Command> > > commandTable;
};


class ScriptInterpreter
{
public:
	ScriptInterpreter(CommandFlagsStack &commandFlagsStack, bool &partiallyFailed, const std::wstring &scriptPath, int recursionDepth = 1);
	~ScriptInterpreter();

	static void AddParams_taskParams(Command &command, const int offset, std::shared_ptr<ParamValidator> noListValidator);

	// return if at least one command was present in the scriptSrc
	bool ExecuteScript(const std::wstring &scriptSrc, bool connect, AutocompleteCollector *collector);
	void Exit() { shouldExit = true; }
	void ResetExit() { shouldExit = false; }
	bool ShouldExit() const { return shouldExit; }
	const ConnectionParams &GetConnectionParams() const { return connectionParams; }

	static std::wstring lastCommand;

	CommandList *GetCommandList();

	CommandFlagsStack &GetCommandFlagsStack(){ return commandFlagsStack; }

private:

	void ReplaceAtFileContent(const std::wstring &scriptSrc, std::wstring &outResult);

	// takes command and breaks it into tokens
	void ParseCommand(const std::wstring &scriptSrc);
	tokenArray tokens;

	// connect to era server
	void Connect();
	void Disconnect();

	// execute a single command, returns true if there is a command in the token array
	bool ExecuteCommand(tokenArray::iterator commandBegin, tokenArray::iterator commandEnd, AutocompleteCollector *collector);

	// group tokens to WORDS/LISTS/FLAGS
	void GroupTokensIntoElements(tokenArray::iterator commandBegin, tokenArray::iterator commandEnd, elementArray &outElements, AutocompleteCollector *collector);

	ConnectionParams connectionParams;

	// list of commands
	CommandList commandList;

	CommandFlagsStack &commandFlagsStack;
	bool &partiallyFailed;

	int recursionDepth;
	bool shouldExit;

	bool connectedByThis;
	void ProcessConnectionParams(ConnectionParams &connectionParams);
	bool ProcessAndApplyInlineFlags(elementArray &elements); // returns true if any flag is present in the command
};

