/**
 * @file TableReader.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "TableReader.h"

#include <Common/scope_exit.h>
#include "EracmdExceptions.h"

TableReader_Const::TableReader_Const(const StringVector &columnNames, const StringVector &prettyColumnNames, const std::vector<StringVector> &rows)
:
	columnNames(columnNames),
	prettyColumnNames(prettyColumnNames),
	rows(rows),
	position(0)
{
}

StringVector TableReader_Const::GetColumns_keyword()
{
	return columnNames;
}

StringVector TableReader_Const::GetColumns_pretty()
{
	return prettyColumnNames;
}

bool TableReader_Const::HasNext()
{
	return (position < rows.size());
}

StringVector TableReader_Const::GetNext()
{
	return rows[position++];
}

TableReader_Api::TableReader_Api(const StringVector &columnNames, const StringVector &prettyColumnNames, Era_TableReader &apiTableReader, const bool prettyFields)
:
	columnNames(columnNames),
	prettyColumnNames(prettyColumnNames),
	apiTableReader(apiTableReader),
	prettyFields(prettyFields)
{
	prepareNextRow();
}

StringVector TableReader_Api::GetColumns_keyword()
{
	return columnNames;
}

StringVector TableReader_Api::GetColumns_pretty()
{
	return prettyColumnNames;
}

bool TableReader_Api::HasNext()
{
	return hasNextRow;
}

StringVector TableReader_Api::GetNext()
{
	StringVector toReturn = std::move(nextRow);
	prepareNextRow();
	return toReturn;
}

void TableReader_Api::prepareNextRow() {
	Era_TableRow *tableRow;
	uint32_t result_readRow = era_ReadRow(&apiTableReader, &tableRow);
	if (result_readRow != 0)
	{
		throw EraCMDException_FromApi(result_readRow);
	}

	hasNextRow = (tableRow != NULL);

	if (hasNextRow)
	{
		scope_exit se_tableRow(std::bind(era_TableRow_delete, tableRow));

		if (prettyFields)
		{
			uint32_t prettyFieldsCount = era_TableRow_GetPrettyFieldsCount(tableRow);
			nextRow.resize(prettyFieldsCount);
			for (uint32_t i = 0; i < prettyFieldsCount; ++i)
			{
				nextRow[i] = era_TableRow_GetPrettyField(tableRow, i);
			}
		}
		else
		{
			uint32_t fieldsCount = era_TableRow_GetFieldsCount(tableRow);
			nextRow.resize(fieldsCount);
			for (uint32_t i = 0; i < fieldsCount; ++i)
			{
				nextRow[i] = era_TableRow_GetField(tableRow, i);
			}
		}
	}
}

