/**
 * @file Command_ClearInfo.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ClearInfo.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/apiUtils.h"
#include "../EracmdExceptions.h"
#include "../Element.h"

void Command_ClearInfo::Execute(elementArray &elements, OutputHandler &out)
{
	// dataType
	std::vector<const ElementWord *> dataTypeList = GetArgument(0)->GetWords();
	DWORD dataType = 0;
	std::map<std::wstring, DWORD> stringEnumMap;
	stringEnumMap[L"threat"] = ERA_INFO_CLEAR_LAST_THREAT;
	stringEnumMap[L"firewall"] = ERA_INFO_CLEAR_LAST_FIREWALL;
	stringEnumMap[L"event"] = ERA_INFO_CLEAR_LAST_EVENT;
	stringEnumMap[L"scan"] = ERA_INFO_CLEAR_LAST_SCANS;
	stringEnumMap[L"custom"] = ERA_INFO_CLEAR_CUSTOM_INFO;
	for (size_t i = 0; i < dataTypeList.size(); ++i)
	{
		std::wstring dataString = dataTypeList[i]->GetString();
		std::transform(dataString.begin(), dataString.end(), dataString.begin(), ::tolower);
		std::map<std::wstring, DWORD>::iterator it = stringEnumMap.find(dataString);
		if (it == stringEnumMap.end())
			throw EraCMDException(CMDERROR_UNKNOWN_CLEARINFO_TYPE_SPECIFIED, dataTypeList[i]->GetString().c_str());		
		dataType |= it->second;
	}

	// clients
	Era_ClientList *clients = ElementToClientList(GetArgument(1));
	scope_exit se_clients(std::bind(era_ClientList_delete, clients));

	uint32_t rc = era_ClearInfo(dataType, clients);
	if (rc != 0)
		throw EraCMDException_FromApi(rc);
}

