/*
 * @file Command_LicenseAdd.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_LicenseAdd.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"


Command_LicenseAdd::Command_LicenseAdd()
:
	Replace(false)
{
}

Command_LicenseAdd::Command_LicenseAdd(bool Replace)
:
	Replace(Replace)
{
}

unsigned int Command_LicenseAdd::GetHelpFullId()
{
	return Replace ? IDS_HELP_LICENSEREPLACE : IDS_HELP_LICENSEADD;
}

unsigned int Command_LicenseAdd::GetHelpShortId()
{
	return Replace ? IDS_HELP_LICENSEREPLACE_SHORT : IDS_HELP_LICENSEADD_SHORT;
}

unsigned int Command_LicenseAdd::GetHelpExampleId()
{
	return Replace ? IDS_HELP_LICENSEREPLACE_EXAMPLE : IDS_HELP_LICENSEADD_EXAMPLE;
}

void Command_LicenseAdd::Execute(elementArray&elements, OutputHandler &out)
{
	std::shared_ptr<Element> param = GetArgument(0);

	std::vector<const ElementWord *> lst = param->GetWords();

	StringVector files;

	Era_LicenseFiles *fileList = era_LicenseFiles_new();
	scope_exit se_fileList(std::bind(&era_LicenseFiles_delete, fileList));

	for (uint32_t i = 0; i < lst.size(); ++i)
	{
		era_LicenseFiles_AddFile(fileList, lst[i]->GetString().c_str());
	}

	uint32_t rc = era_AddLicense(fileList, Replace);
	if (rc != ERAERROR_SUCCESS)
		throw EraCMDException_FromApi(rc);
}

