/**
 * @file Command_Password.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_Password.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/apiUtils.h"
#include "../util/stringUtils.h"
#include "../ConsoleKeyboardInput.h"
#include "../EracmdExceptions.h"

Command_Password::Command_Password(ScriptInterpreter *scriptInterpreter) : scriptInterpreter(scriptInterpreter)
{
}

void Command_Password::Execute(elementArray &elements, OutputHandler &out)
{
	// type
	Era_PasswordType passType;
	std::wstring typeStr = GetArgumentAsString(0);
	if (_wcsicmp(typeStr.c_str(), L"client") == 0)
		passType = ERA_PASSWORD_TYPE__INSTALLER;
	else if (_wcsicmp(typeStr.c_str(), L"replication") == 0)
		passType = ERA_PASSWORD_TYPE__REPLICATION;
	else if (_wcsicmp(typeStr.c_str(), L"installer") == 0)
		passType = ERA_PASSWORD_TYPE__INSTALLER;
	else if (_wcsicmp(typeStr.c_str(), L"currentuser") == 0)
		passType = ERA_PASSWORD_TYPE__CURRENTUSER;

	// passwords
	std::wstring newPassword, oldPassword;
	std::shared_ptr<Element> oldPassElement = GetArgument(1);
	if (oldPassElement == NULL)
	{
		// ask for passwords
		std::wstring newPasswordConfirm = L"rgwergasdqwqsadfagf";
		unsigned int msgs[3] = {IDS_ENTER_OLD_PASSWORD, IDS_ENTER_NEW_PASSWORD, IDS_ENTER_NEW_PASSWORD_AGAIN};
		std::wstring *targets[3] = {&oldPassword, &newPassword, &newPasswordConfirm};

		for (int i = 0; i < 3; ++i)
		{
			ConsoleKeyboardInput keyInput(*scriptInterpreter, true);
			std::wstring passwordMessage = LoadStringFromResource(msgs[i]);
			std::wcout << passwordMessage << L" ";
			keyInput.GetInput();
			*targets[i] = keyInput.GetLine();
			std::wcout << std::endl;
		}
		if (newPasswordConfirm != newPassword)
			throw EraCMDException(CMDERROR_PASSWORD_NOT_MATCHING);
	}
	else
	{
		oldPassword = GetArgumentAsString(1);
		newPassword = GetArgumentAsString(2);
	}

	uint32_t result = era_ChangePassword(passType, oldPassword.c_str(), newPassword.c_str());
	if (result != 0)
		throw EraCMDException_FromApi(result);
}

