/**
 * @file Command_PolicyDelete.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_PolicyDelete.h"

#include <Common/era_api/err_codes.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_PolicyDelete::Execute(elementArray &elements, OutputHandler &out)
{
	uint32_t deletedPolicyId = 0;
	{
		const std::wstring elementStr = GetArgument(0)->GetString();

		try
		{
			deletedPolicyId = ::WstringToUint(elementStr);
		}
		catch (...)
		{
			throw EraCMDException(CMDERROR_NUMBER_EXPECTED_FOR_POLICY_ID, elementStr.c_str());
		}
	}

	PolicyReplacementPtr newParentPolicyForChildPolicies(NULL, era_PolicyReplacement_delete);
	PolicyReplacementPtr newPolicyForPrimaryClients(NULL, era_PolicyReplacement_delete);
	PolicyReplacementPtr newPolicyForReplicatedClients(NULL, era_PolicyReplacement_delete);
	PolicyReplacementPtr newDefaultPolicyForPrimaryClients(NULL, era_PolicyReplacement_delete);
	PolicyReplacementPtr newDefaultPolicyForLowerServers(NULL, era_PolicyReplacement_delete);

	FillReplacement(1, newParentPolicyForChildPolicies);
	FillReplacement(2, newPolicyForPrimaryClients);
	FillReplacement(3, newPolicyForReplicatedClients);
	FillReplacement(4, newDefaultPolicyForPrimaryClients);
	FillReplacement(5, newDefaultPolicyForLowerServers);

	bool deleteWholeBranch = false;
	if (std::shared_ptr<Element> element = GetArgument(6))
	{
		if (::ToBool(element->GetString()))
		{
			deleteWholeBranch = true;
		}
	}

	uint32_t result_policyDelete = era_Policy_delete(deletedPolicyId, newParentPolicyForChildPolicies.get(), newPolicyForPrimaryClients.get(), newPolicyForReplicatedClients.get(), newDefaultPolicyForPrimaryClients.get(), newDefaultPolicyForLowerServers.get(), deleteWholeBranch);
	if (result_policyDelete != 0)
	{
		throw EraCMDException_FromApi(result_policyDelete);
	}
}

