/**
 * @file Command_PolicyUpdate.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_PolicyUpdate.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../EracmdExceptions.h"

void Command_PolicyUpdate::Execute(elementArray &elements, OutputHandler &out)
{
	Era_PolicyParams *params = era_PolicyParams_new();
	scope_exit se_query(std::bind(era_PolicyParams_delete, params));

	int nPolicyId = GetNumParam(0);
	bool bAtLeastOneParam = FillPolicyParams(1, true, *params);
	if (!bAtLeastOneParam)
		throw EraCMDException(CMDERROR_POLICY_UPDATE_MISSING_PARAMS);

	PolicyReplacementPtr newPolicyForReplicatedClients(NULL, era_PolicyReplacement_delete);
	FillReplacement(9, newPolicyForReplicatedClients);
	PolicyReplacementPtr newDefaultPolicyForLowerServers(NULL, era_PolicyReplacement_delete);
	FillReplacement(10, newDefaultPolicyForLowerServers);
	PolicyReplacementPtr newDefaultPolicyForPrimaryClients(NULL, era_PolicyReplacement_delete);
	FillReplacement(11, newDefaultPolicyForPrimaryClients);

	uint32_t result_policyUpdate = era_Policy_update(nPolicyId, params, newPolicyForReplicatedClients.get(), newDefaultPolicyForLowerServers.get(), newDefaultPolicyForPrimaryClients.get());
	if (result_policyUpdate != 0)
	{
		throw EraCMDException_FromApi(result_policyUpdate);
	}
}
