/**
 * @file Command_ReportCreate.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ReportCreate.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_ReportCreate::Execute(elementArray &elements, OutputHandler &out)
{
	Era_ReportTemplateParams *params = era_ReportTemplateParams_new();
	scope_exit se_query(std::bind(era_ReportTemplateParams_delete, params));

	FillReportParams(0, *params);

	uint32_t newReportId = -1;
	uint32_t result_reportCreate = era_ReportTemplate_create(&newReportId, params);
	if (result_reportCreate != 0)
	{
		throw EraCMDException_FromApi(result_reportCreate);
	}

	StringVector header(1);
	header[0] = L"report_id";
	StringVector prettyHeader(1);
	prettyHeader[0] = LoadStringFromResource(IDS_GROUP_ID);
	std::vector<StringVector> rows;
	StringVector data(1);
	data[0] = std::to_wstring((long long)newReportId);
	rows.push_back(data);
	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}
