/**
 * @file Command_Request.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_Request.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/apiUtils.h"
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"
#include "../Element.h"


void Command_Request::Execute(elementArray &elements, OutputHandler &out)
{
	// dataType
	std::vector<const ElementWord *> dataTypeList = GetArgument(0)->GetWords();
	DWORD dataType = 0;
	std::map<std::wstring, DWORD> stringEnumMap;
	stringEnumMap[L"sysinspector"] = ERA_DATA_SYSINSPECTOR;
	stringEnumMap[L"configuration"] = ERA_DATA_CONFIGURATION;
	stringEnumMap[L"protection_status"] = ERA_DATA_STATUS;
	stringEnumMap[L"protection_features"] = ERA_DATA_FEATURES;
	stringEnumMap[L"system_information"] = ERA_DATA_SYSTEM_INFORMATION;
	for (size_t i = 0; i < dataTypeList.size(); ++i)
	{
		std::wstring dataString = dataTypeList[i]->GetString();
		std::transform(dataString.begin(), dataString.end(), dataString.begin(), ::tolower);
		std::map<std::wstring, DWORD>::iterator it = stringEnumMap.find(dataString);
		if (it == stringEnumMap.end())
		{
			throw EraCMDException(CMDERROR_UNKNOWN_DATA_TYPE_REQUESTED, dataTypeList[i]->GetString().c_str());
		}
		dataType |= it->second;
	}

	// clients
	Era_ClientList *clients = ElementToClientList(GetArgument(1));
	scope_exit se_clients(std::bind(era_ClientList_delete, clients));

	// date
	Era_SysinspectorParams *siParams = NULL;
	scope_exit se_siParams(std::bind(era_SysinspectorParams_delete, std::ref(siParams)));

	std::shared_ptr<Element> dateArg = GetArgument(2);
	std::shared_ptr<Element> snapshotArg = GetArgument(3);
	if (dateArg || snapshotArg)
	{
		siParams = era_SysinspectorParams_new();
		if (dateArg != NULL)
		{
			FILETIME compareTo_fileTime = ToFileTime(dateArg->GetString());

			ULARGE_INTEGER compareTo;

			compareTo.LowPart = compareTo_fileTime.dwLowDateTime;
			compareTo.HighPart = compareTo_fileTime.dwHighDateTime;

			era_SysinspectorParams_SetCompareToDate(siParams, compareTo.QuadPart);
		}
		if (snapshotArg != NULL)
		{
			std::wstring snapString = snapshotArg->GetString();
			if (_wcsicmp(snapString.c_str(), L"si_snapshot") != 0)
				throw EraCMDException(CMDERROR_COMMAND_UNEXPECTED_ARGUMENT, snapString.c_str(), L"si_snapshot");
			era_SysinspectorParams_SetSnapshot(siParams, true);
		}
	}

	uint32_t rc = era_RequestData(dataType, clients, siParams);
	if (rc != 0)
	{
		throw EraCMDException_FromApi(rc);
	}
}

