/**
 * @file Command_RunScript.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_RunScript.h"

#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "..\EracmdExceptions.h"
#include "..\Element.h"
#include "..\FileLoader.h"
#include "..\ScriptInterpreter.h"

std::vector<std::wstring> Command_RunScript::scriptFileStack;

Command_RunScript::Command_RunScript(CommandFlagsStack &commandFlagsStack, bool &partiallyFailed, int recursionDepth) : commandFlagsStack(commandFlagsStack), partiallyFailed(partiallyFailed), recursionDepth(recursionDepth)
{

}

std::wstring Command_RunScript::GetCurrentFileName()
{
	if (scriptFileStack.empty()) return L"";
	return *scriptFileStack.rbegin();
}

void Command_RunScript::Execute(elementArray &elements, OutputHandler &out)
{
	std::wstring fileName = GetArgument(0)->GetString();

	// check depth
	if (recursionDepth > 32)
		throw EraCMDException(CMDERROR_SCRIPT_TOO_NESTED, fileName.c_str());
	// check stack
	std::wstring fileNameUp = fileName;
	std::transform(fileNameUp.begin(), fileNameUp.end(), fileNameUp.begin(), ::toupper);
	if (std::find(scriptFileStack.begin(), scriptFileStack.end(), fileNameUp) != scriptFileStack.end())
	{
		throw EraCMDException(CMDERROR_SCRIPT_INFINITE_LOOP, fileName.c_str());
	}
	scriptFileStack.push_back(fileNameUp);
	scope_exit se(std::bind(&std::vector<std::wstring>::pop_back, &scriptFileStack));

	FileLoader fileLoader(fileName);
	std::wstring script = fileLoader.Load();

	std::wstring scriptFullPath;
	wchar_t *buffer;
	if ((buffer = ::_wfullpath(NULL, fileName.c_str(), 0)) != NULL)
	{
		scriptFullPath = buffer;
		free(buffer);
	}
	else
	{
		scriptFullPath = fileName;
	}

	ScriptInterpreter si(commandFlagsStack, partiallyFailed, scriptFullPath, recursionDepth + 1);
	si.ExecuteScript(script, false, NULL);
}

