/**
 * @file Command_ShowTables.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ShowTables.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"

void Command_ShowTables::Execute(elementArray &elements, OutputHandler &out)
{
	Era_TablesInfo *tablesInfo;

	uint32_t result_getTablesInfo = era_GetTablesInfo(&tablesInfo);
	if (result_getTablesInfo != 0)
	{
		throw EraCMDException_FromApi(result_getTablesInfo);
	}
	scope_exit se_tablesInfo(std::bind(era_TablesInfo_delete, tablesInfo));

	StringVector header(2);
	header[0] = L"keyword";
	header[1] = L"name";

	StringVector prettyHeader(2);
	prettyHeader[0] = LoadStringFromResource(IDS_SHOWTABLES_COLNAME_KEYWORD);
	prettyHeader[1] = LoadStringFromResource(IDS_SHOWTABLES_COLNAME_NAME);

	uint32_t tableNamesCount = era_TablesInfo_GetTableNamesCount(tablesInfo);
	std::vector<StringVector> rows(tableNamesCount);

	assert(era_TablesInfo_GetTableNamesCount(tablesInfo) == era_TablesInfo_GetPrettyTableNamesCount(tablesInfo));
	for (uint32_t i = 0; i < tableNamesCount; ++i)
	{
		StringVector row(2);
		row[0] = era_TablesInfo_GetTableName(tablesInfo, i);
		row[1] = era_TablesInfo_GetPrettyTableName(tablesInfo, i);
		rows[i] = std::move(row);
	}

	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

