/**
 * @file Command_TaskConfig.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_TaskConfig.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/apiUtils.h"
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_TaskConfig::Execute(elementArray &elements, OutputHandler &out)
{
	Era_TaskParams *params = era_TaskParams_new();
	scope_exit se_query(std::bind(era_TaskParams_delete, params));

	std::wstring configFilePath = GetArgument(0)->GetString();

	FillTaskParams(1, *params);

	Era_ClientList *clients = ElementToClientList(GetArgument(1));
	scope_exit se_clients(std::bind(era_ClientList_delete, clients));

	uint32_t taskId = -1;
	uint32_t result_taskConfig = era_Task_config(&taskId, configFilePath.c_str(), clients, params);
	if (result_taskConfig != 0)
	{
		throw EraCMDException_FromApi(result_taskConfig);
	}

	StringVector header(1);
	header[0] = L"task_id";
	StringVector prettyHeader(1);
	prettyHeader[0] = LoadStringFromResource(IDS_TASK_ID);
	std::vector<StringVector> rows;
	StringVector data(1);
	data[0] = std::to_wstring((long long)taskId);
	rows.push_back(data);
	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

