/**
 * @file ConsoleColors_Win.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "ConsoleColors_Win.h"

typedef BOOL (WINAPI *funcGetConsoleScreenBufferInfoEx) (HANDLE, PCONSOLE_SCREEN_BUFFER_INFOEX);
typedef BOOL (WINAPI *funcSetConsoleScreenBufferInfoEx) (HANDLE, PCONSOLE_SCREEN_BUFFER_INFOEX);


ConsoleColors_Win::ConsoleColors_Win()
:
	console_cout(::GetStdHandle(STD_OUTPUT_HANDLE)),
	console_cerr(::GetStdHandle(STD_ERROR_HANDLE))
{
	enableColors = true;
	CONSOLE_SCREEN_BUFFER_INFO info;

	if (GetConsoleScreenBufferInfo(console_cout, &info))
	{
		defaultTextColor_cout = (info.wAttributes & (FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY));
	}
	else
	{
		defaultTextColor_cout = 7;
	}

	if (GetConsoleScreenBufferInfo(console_cerr, &info))
	{
		defaultTextColor_cerr = (info.wAttributes & (FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY));
	}
	else
	{
		defaultTextColor_cerr = 7;
	}

	originalBlueColor = SetBlueColorRGB(RGB(36, 36, 36));
}

ConsoleColors_Win::~ConsoleColors_Win()
{
	SetOrigColors();
}

COLORREF ConsoleColors_Win::SetBlueColorRGB(COLORREF color)
{
	COLORREF result = RGB(0, 0, 128);
	// try if we can use GetConsoleScreenBufferInfoEx
	backGreyColor = 0;
	HINSTANCE hDLL = LoadLibrary(L"kernel32.dll");
	if (hDLL)
	{
		funcGetConsoleScreenBufferInfoEx GetFunc = (funcGetConsoleScreenBufferInfoEx) GetProcAddress(hDLL, "GetConsoleScreenBufferInfoEx");
		funcSetConsoleScreenBufferInfoEx SetFunc = (funcSetConsoleScreenBufferInfoEx) GetProcAddress(hDLL, "SetConsoleScreenBufferInfoEx");
		if (GetFunc && SetFunc)
		{
			CONSOLE_SCREEN_BUFFER_INFOEX info2;
			info2.cbSize = sizeof(info2);
			if (GetFunc(console_cout, &info2))
			{
				backGreyColor = 1; // BLUE
				result = info2.ColorTable[backGreyColor];
				info2.ColorTable[backGreyColor] = color;
				info2.srWindow.Bottom++;
				info2.srWindow.Right++;
				if (FALSE == SetFunc(console_cout, &info2))
				{
					backGreyColor = 0;
				}
			}
		}
	}
	return result;
}

void ConsoleColors_Win::SetColoring(bool enable)
{
	enableColors = enable;
}

void ConsoleColors_Win::SetTextColor(std::wostream &os, const Color &color)
{
	HANDLE console;
	int defaultTextColor;
	if (os == std::wcout)
	{
		console = console_cout;
		defaultTextColor = defaultTextColor_cout;
	}
	else if (os == std::wcerr)
	{
		console = console_cerr;
		defaultTextColor = defaultTextColor_cerr;
	}
	else
	{
		return;
	}

	if ((console == NULL) || (console == INVALID_HANDLE_VALUE))
	{
		return;
	}

	if (!enableColors)
	{
		::SetConsoleTextAttribute(console, defaultTextColor);
		return;
	}

	if (&color == &DEFAULT)
	{
		::SetConsoleTextAttribute(console, defaultTextColor);
	}
	if (&color == &DEFAULT_LIGHTBACK)
	{
		::SetConsoleTextAttribute(console, defaultTextColor | (backGreyColor<<4));
	}

	if (&color == &BLACK)
	{
		::SetConsoleTextAttribute(console, 0);
	}
	if (&color == &BLUE)
	{
		::SetConsoleTextAttribute(console, 1);
	}
	if (&color == &GREEN)
	{
		::SetConsoleTextAttribute(console, 2);
	}
	if (&color == &GREEN_LIGHTBACK)
	{
		::SetConsoleTextAttribute(console, 2 | (backGreyColor<<4));
	}
	if (&color == &AQUA)
	{
		::SetConsoleTextAttribute(console, 3);
	}
	if (&color == &RED)
	{
		::SetConsoleTextAttribute(console, 4);
	}
	if (&color == &PURPLE)
	{
		::SetConsoleTextAttribute(console, 5);
	}
	if (&color == &YELLOW)
	{
		::SetConsoleTextAttribute(console, 6);
	}
	if (&color == &LIGHT_GRAY)
	{
		::SetConsoleTextAttribute(console, 7);
	}
	if (&color == &GRAY)
	{
		::SetConsoleTextAttribute(console, 8);
	}
	if (&color == &LIGHT_BLUE)
	{
		::SetConsoleTextAttribute(console, 9);
	}
	if (&color == &LIGHT_GREEN)
	{
		::SetConsoleTextAttribute(console, 10);
	}
	if (&color == &LIGHT_AQUA)
	{
		::SetConsoleTextAttribute(console, 11);
	}
	if (&color == &LIGHT_RED)
	{
		::SetConsoleTextAttribute(console, 12);
	}
	if (&color == &LIGHT_PURPLE)
	{
		::SetConsoleTextAttribute(console, 13);
	}
	if (&color == &LIGHT_YELLOW)
	{
		::SetConsoleTextAttribute(console, 14);
	}
	if (&color == &WHITE)
	{
		::SetConsoleTextAttribute(console, 15);
	}
}

void ConsoleColors_Win::SetOrigColors()
{
	::SetConsoleTextAttribute(console_cout, defaultTextColor_cout);
	::SetConsoleTextAttribute(console_cerr, defaultTextColor_cerr);
	SetBlueColorRGB(originalBlueColor);
}

