/**
 * @file debugUtils.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"

#ifdef _DEBUG

#include "Element.h"
#include "Token.h"

void DumpTokenizedCommand(tokenArray &Tokens)
{
	std::wcout << std::endl;
	std::wstring cmd;
	for (size_t i = 0; i < Tokens.size(); ++i)
	{
		std::shared_ptr<Token> token = Tokens[i];
		std::wcout << token->text << std::endl;
		if (!cmd.empty())
			cmd.push_back(L' ');
		cmd += token->text;
		std::wstring tokenTypeString = Token::GetTokenTypeAsString(token->type);
		std::wcout << tokenTypeString << std::endl;
	}
	std::wcout << std::endl << cmd << std::endl << std::endl;
}


std::wstring ElementArrayToDebugString(elementArray &inElements, bool detailed)
{
	std::wstring basicOut;
	std::wstring detailedOut;
	for (size_t i = 0; i < inElements.size(); ++i)
	{
		if (i != 0) basicOut += L" ";
		if (inElements[i]->type == Element::element_wordlist)
		{
			detailedOut += L"WORDLIST:\n";
			std::shared_ptr<ElementWordList> wordList = std::static_pointer_cast<ElementWordList>(inElements[i]);
			for (size_t j = 0; j < wordList->wordList.size(); ++j)
			{
				std::shared_ptr<ElementWord> word = wordList->wordList[j];

				detailedOut += L"  WORD: " + word->GetString() + L"\n";

				if (j != 0) basicOut += L",";
				basicOut += word->GetString();

				for (size_t k = 0; k < word->tokenList.size(); ++k)
				{
					detailedOut += L"    TOKEN: " + word->tokenList[k]->text + L" " + Token::GetTokenTypeAsString(word->tokenList[k]->type) + L"\n";
				}
			}
		}
		else
		{
			std::shared_ptr<ElementWord> word = std::static_pointer_cast<ElementWord>(inElements[i]);
			detailedOut += L"WORD: " + word->GetString() + L"\n";
			basicOut += word->GetString();
			for (size_t k = 0; k < word->tokenList.size(); ++k)
			{
				detailedOut += L"  TOKEN: " + word->tokenList[k]->text + L" " + Token::GetTokenTypeAsString(word->tokenList[k]->type) + L"\n";
			}
		}
	}
	return detailed ? basicOut + L"\n" + detailedOut : basicOut;
}
#endif

