/**
 * @file PagingStream.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

class PagingStreambuf : public std::wstreambuf
{

public:
	PagingStreambuf(std::wstreambuf &dest, std::function<void()> hintPrintingFunc, std::function<void()> headerPrintingFunc);
	virtual int_type overflow (int_type ch) override;
	virtual int_type underflow() override;
	virtual int sync() override;
	virtual std::wstreambuf *setbuf(wchar_t *p, std::streamsize len) override;

private:
	void SetInitialCursorPosition();

	void PrintHint();
	void PrintHeader();

private:
	std::wstreambuf &dest;
	std::function<void()> hintPrintingFunc;
	std::function<void()> headerPrintingFunc;

	bool printingHint;
	bool printingHeader;

	bool exited;
	bool disabled;

	SHORT initialCursorPosition;

	HANDLE stdOutputHandle;
};

class StreamModifier_Paging
{

public:
	StreamModifier_Paging(std::wostream &origStream, const bool enablePaging, std::function<void()> hintPrintingFunc, std::function<void()> headerPrintingFunc = []{});
	~StreamModifier_Paging();

private:
	std::wostream &origStream;
	bool pagingEnabled;

	std::wstreambuf *prevRdbuf;

};

