/**
 * @file TextWidth_Console.h
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#pragma once

#include "TextWidth.h"

struct FontWidthSection
{
	uint16_t range_first;
	uint16_t range_last;
	uint16_t width;

	FontWidthSection(const uint16_t range_first, const uint16_t range_last, const uint16_t width)
	:	range_first(range_first),
		range_last(range_last),
		width(width)
	{}

	FontWidthSection() {};
};

struct FontWidthTable
{
	typedef std::wstring Font;
	typedef std::vector<FontWidthSection> Sections;

	typedef std::map<Font, Sections> WidthsForFonts;
	WidthsForFonts widthsForFonts;
};

class TextWidth_Console : public TextWidth
{

public:
	TextWidth_Console();

private:
	void LoadTableFromResources();
	std::wstring GetCurrentFont();
	void ExtractCurrentFontWidths();

	void LoadTableFromStream(std::istream &is);

	#ifdef _DEBUG
		void LoadTableFromFile();
		void SaveTableToFile();
		void EnsureTableHasCurrentFontInfo();
	#endif

public:
	virtual size_t GetWidth(const std::wstring &str) const;
	virtual std::wstring AlignTo(const std::wstring &str, const size_t width) const;

private:

	FontWidthTable table;

	static const uint32_t charsCount = 0x10000;
	SHORT charWidths[charsCount];

};

